/*
	Crossing
	Copyright (C) 2007 Michal Vaner (michal.vaner@kdemail.net)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	he Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#include "behav.h"
#include "globals.h"
#include "load.h"
#include "point.h"
#include "semaphore_group.h"
#include "indexer.h"
#include "vehicle.h"

#include <QtAlgorithms>
#include <QCoreApplication>

VehicState::VehicState() :
	speed(50),
	priority(1),
	cont(NULL)
{
}

Behavior::~Behavior() {
}

Condition::Condition(const QDomElement &node, Point *point) :
	cleaner(this)
{
	const QDomNodeList &children(node.childNodes());
	std::auto_ptr<SpeedBehav> speed(NULL);
	std::auto_ptr<PriorityBehav> priority(NULL);
	std::auto_ptr<ContBehav> cont(NULL);
	for(int i = 0; i < children.count(); ++i) {
		const QDomNode &ch(children.at(i));
		if(!ch.isElement())
			continue;
		const QDomElement ech(ch.toElement());
		if(ech.namespaceURI() != NS)
			continue;
		if(ech.nodeName() == "vehicle")
			conditions << new VehicCond(ech, point);
		else if(ech.nodeName() == "from")
			conditions << new FromCond(ech, point);
		else if(ech.nodeName() == "speed")
			speed.reset(new SpeedBehav(ech));
		else if(ech.nodeName() == "cont") {
			if(cont.get())
				cont->addMore(ech, point);
			else
				cont.reset(new ContBehav(ech, point));
		} else if(ech.nodeName() == "semaphore")
			behaviors << new SemaphoreBehav(ech, point->indexer());
		else if(ech.nodeName() == "priority")
			priority.reset(new PriorityBehav(ech));
	}
	if(speed.get())
		behaviors << speed.release();
	if(priority.get())
		behaviors << priority.release();
	if(cont.get())
		behaviors << cont.release();
}

Condition::Cleaner::~Cleaner() {
	qDeleteAll(mCondition->conditions);
	qDeleteAll(mCondition->behaviors);
}

VehicState Condition::processVehicle(const VehicState &state, const Vehicle &vehicle) const {
	if(match(vehicle)) {
		VehicState st(state);
		foreach(Behavior *b, behaviors)
			st = b->processVehicle(st, vehicle);
		foreach(Condition *c, conditions)
			st = c->processVehicle(st, vehicle);
		return st;
	} else
		return state;
}

bool RootCond::match(const Vehicle &) const {
	return true;
}

RootCond::RootCond(const QDomElement &node, Point *point) :
	Condition(node, point)
{
	behaviors.prepend(new SpeedBehav());//Add the defaults - they will be overwritten
	behaviors.prepend(new ContBehav(point));
	behaviors.prepend(new PriorityBehav());
}

FromCond::FromCond(const QDomElement &node, Point *point) :
	Condition(node, point)
{
	QString id(node.attribute("id"));
	if(!(line = point->inLine(id)))
		throw XMLLoadDataE(QCoreApplication::translate("xml_errors", "From condition ID invalid"));
}

bool FromCond::match(const Vehicle &vehicle) const {
	return vehicle.actLine() == line;
}

VehicCond::VehicCond(const QDomElement &node, Point *point) :
	Condition(node, point),
	type(vehicTypes().value(node.attribute("type")))
{
	if(!type)
		throw XMLLoadDataE(QCoreApplication::translate("xml_errors", "Unknown vehicle type %1").arg(node.attribute("type")));
}

bool VehicCond::match(const Vehicle &vehicle) const {
	return vehicle.vtype == type;
}

ContBehav::ContBehav(const QDomElement &node, Point *point) :
	max(0)
{
	addMore(node, point);
}

ContBehav::ContBehav(Point *point) {
	QList<Line *> lines = point->allOutLines();
	max = lines.count();
	int i = 0;
	foreach(Line *l, lines) {
		Cont c;
		c.line = l;
		c.start = i++;
		c.len = 1;
		conts << c;
	}
}

void ContBehav::addMore(const QDomElement &node, Point *point) {
	int probab = 1;
	QString attr(node.attribute("probability"));
	if(!attr.isEmpty()) {
		bool ok;
		probab = attr.toInt(&ok);
		if(!ok)
			throw XMLLoadDataE(QCoreApplication::translate("xml_errors", "Invalid priority"));
	}
	Cont cont;
	cont.start = max;
	cont.len = probab;
	if(!(cont.line = point->line(node.text())))
		throw XMLLoadDataE(QCoreApplication::translate("xml_errors", "Cont behavior ID invalid"));
	if(!probab)
		return;
	max += probab;
	conts << cont;
}

VehicState ContBehav::processVehicle(const VehicState &state, const Vehicle &) const {
	VehicState st(state);
	if(max) {
		int c = random() % max;
		int l = 0, r = conts.count(), p = (l + r) / 2;
		while(!((conts[p].start <= c) && (conts[p].start + conts[p].len > c))) {
			if(c < conts[p].start)
				r = p - 1;
			else
				l = p + 1;
			p = (l + r) / 2;
		}
		st.cont = conts[p].line;
	} else
		st.cont = NULL;
	return st;
}

SemaphoreBehav::SemaphoreBehav(const QDomElement &node, Indexer *indexer) {
	group = indexer->semaphoreGroups().value(node.attribute("group"));
	if(!group)
		throw XMLLoadDataE(QCoreApplication::translate("xml_errors", "Invalid semaphore group"));
	const QDomNodeList &inters(node.elementsByTagName("interval"));
	for(int i = 0; i < inters.count(); i++) {
		const QDomElement &inter(inters.at(i).toElement());
		bool ok1, ok2;
		int start = inter.attribute("start").toInt(&ok1), end = inter.attribute("end").toInt(&ok2);
		if(!ok1 || !ok2)
			throw XMLLoadDataE(QCoreApplication::translate("xml_errors", "Invalid semaphore interval"));
		Interval interval;
		interval.start = start;
		interval.end = end;
		intervals << interval;
	}
	qSort(intervals);
}

VehicState SemaphoreBehav::processVehicle(const VehicState &state, const Vehicle &) const {
	VehicState s(state);
	s.semaphores << this;
	return s;
}

bool SemaphoreBehav::isOpen() const {
	int actPos = group->pos();
	foreach(const Interval &interval, intervals) {
		if((interval.start <= actPos) && (interval.end >= actPos))
			return true;
	}
	return false;
}

SpeedBehav::SpeedBehav(const QDomElement &node) :
	speed(node.text().toInt())
{
	if(!speed)
		throw XMLLoadDataE(QCoreApplication::translate("xml_errors", "Speed not set"));
}

SpeedBehav::SpeedBehav() :
	speed(50)
{
}

VehicState SpeedBehav::processVehicle(const VehicState &state, const Vehicle &) const {
	VehicState st(state);
	st.speed = speed;
	return st;
}

PriorityBehav::PriorityBehav(const QDomElement &node) :
	priority(node.text().toInt())
{
	if(!priority)
		throw XMLLoadDataE(QCoreApplication::translate("xml_errors", "Priority not set"));
}

PriorityBehav::PriorityBehav() :
	priority(1)
{
}

VehicState PriorityBehav::processVehicle(const VehicState &state, const Vehicle &) const {
	VehicState st(state);
	st.priority = priority;
	return st;
}
