/*
	Crossing
	Copyright (C) 2007 Michal Vaner (michal.vaner@kdemail.net)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	he Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "cross.h"
#include "crossget.h"
#include "load.h"
#include "indexer.h"

#include <QActionGroup>
#include <QScrollArea>
#include <QFileDialog>
#include <QMessageBox>
#include <QTimer>
#include <QTextBrowser>

Cross::Cross() :
	indexer(NULL)
{
	setupUi(this);
	QActionGroup *speedGrp = new QActionGroup(this);//This will free itself, when the window is freed
	speedGrp->setExclusive(true);
	speedGrp->addAction(slowSpAc);
	speedGrp->addAction(normalSpAc);
	speedGrp->addAction(fastSpAc);
	QActionGroup *trafGrp = new QActionGroup(this);
	trafGrp->addAction(lightTrafAc);
	trafGrp->addAction(moderateTrafAc);
	trafGrp->addAction(heavyTrafAc);
	trafGrp->addAction(veryHeavyTrafAc);
	QScrollArea *area = new QScrollArea(this);
	area->setWidget(crossget = new Crossget(this));
	setCentralWidget(area);
	connect(drawStructAc, SIGNAL(toggled(bool)), crossget, SLOT(setDrawStruct(bool)));
	connect(slowSpAc, SIGNAL(triggered()), SLOT(updateSpeed()));
	connect(normalSpAc, SIGNAL(triggered()), SLOT(updateSpeed()));
	connect(fastSpAc, SIGNAL(triggered()), SLOT(updateSpeed()));
	connect(stopAc, SIGNAL(triggered()), crossget, SLOT(update()), Qt::QueuedConnection);//call it after it is done
	connect(lightTrafAc, SIGNAL(triggered()), SLOT(updateTraffic()));
	connect(moderateTrafAc, SIGNAL(triggered()), SLOT(updateTraffic()));
	connect(heavyTrafAc, SIGNAL(triggered()), SLOT(updateTraffic()));
	connect(veryHeavyTrafAc, SIGNAL(triggered()), SLOT(updateTraffic()));
}

void Cross::on_openAc_triggered() {
	QString fn = QFileDialog::getOpenFileName(this, tr("Select the crossing description file"), "", tr("XML files(*.xml)"));
	if(!fn.isEmpty()) {
		try {
			indexer = loadXmlFile(fn, crossget);
			connect(pauseAc, SIGNAL(triggered()), indexer, SLOT(pause()));
			connect(stopAc, SIGNAL(triggered()), indexer, SLOT(stop()));
			connect(startAc, SIGNAL(triggered()), indexer, SLOT(start()));
			startAc->setEnabled(true);
			pauseAc->setEnabled(false);
			stopAc->setEnabled(false);
			updateSpeed();
			updateTraffic();
		}
		catch (const XMLLoadFormatE &) {
			showErr(tr("Syntax error in the XML file"));
		}
		catch (const XMLLoadForeignE &) {
			showErr(tr("This XML does not describe a crossing"));
		}
		catch (const XMLLoadFileNotFoundE &) {
			showErr(tr("The XML file not found"));
		}
		catch (const XMLLoadDataE &e) {
			showErr(tr("Nonsense in the XML file (%1)").arg(e.text));
		}
		catch (const ImageNotFoundE &e) {
			showErr(tr("Image file could not be found (%1)").arg(e.filename));
		}
		catch (const LoadUnknownE &) {
			showErr(tr("Unknown error"));//Can not happen
		}
	}
}

void Cross::showErr(const QString &msg) {
	QMessageBox::critical(this, tr("Error loading the crossing"), msg, QMessageBox::Ok, QMessageBox::NoButton);
}

void Cross::on_startAc_triggered() {
	pauseAc->setEnabled(true);
	stopAc->setEnabled(true);
	startAc->setEnabled(false);
}

void Cross::on_pauseAc_triggered() {
	pauseAc->setEnabled(false);
	stopAc->setEnabled(true);
	startAc->setEnabled(true);
}

void Cross::on_stopAc_triggered() {
	pauseAc->setEnabled(false);
	stopAc->setEnabled(false);
	startAc->setEnabled(true);
}

void Cross::updateSpeed() {
	if(indexer)
		indexer->setSpeed((!!normalSpAc->isChecked()) + 2 * (!!fastSpAc->isChecked()));
}

void Cross::updateTraffic() {
	if(indexer)
		indexer->setTraffic((!!moderateTrafAc->isChecked()) + 2 * (!!heavyTrafAc->isChecked()) + 3 * (!!veryHeavyTrafAc->isChecked()));
}

void Cross::on_uguideAc_triggered() {
	showHelp("user");
}

void Cross::on_mguideAc_triggered() {
	showHelp("modeler");
}

void Cross::showHelp(const QString &page) {
	QTextBrowser *bs = new QTextBrowser();
	bs->setAttribute(Qt::WA_DeleteOnClose);
	bs->setWindowFlags(Qt::Window);
	bs->setSearchPaths(QStringList() << tr("://help/", "where to fend the dox"));
	bs->setSource(tr("%1.html", "Where to get the help").arg(page));
	bs->setWindowTitle(tr("Crossing help"));
	bs->showMaximized();
}
