/*
	Crossing
	Copyright (C) 2007 Michal Vaner (michal.vaner@kdemail.net)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	he Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#ifndef LEVELS_H
#define LEVELS_H

#include <QObject>
#include <QList>
#include <QHash>
#include <QTime>

class Point;
class Line;
class Indexer;
class SemaphoreGroup;
class QTimer;

class Indexable {
	public:
		enum Type {
			Vehicle,//Something that can move
			Structural,//Lines and points - logical structure of the crossing
			Object,//Something drawn on the map
			Invisible//Somethinc that is not visible on the map
		};
	private:
		int mIndex;
		Type mType;
		friend class Indexer;
		Indexer *mIndexer;//My indexer, it will place itself here
	public:
		virtual int index() const;
		Type type() const { return mType; }
		Indexer *indexer() { return mIndexer; }
		virtual ~Indexable();
		Indexable(Type type, int index = 0);
		void changeIndex(int newIndex);
};

class Invisible : public Indexable {
	public:
		Invisible() : Indexable(Indexable::Invisible) {};
};

class Indexer : public QObject {
	Q_OBJECT;
	public:
		template<class T, class TP>
		static QList<T> filter(const QList<T> &list, TP type) {
			QList<T> result;
			foreach(T i, list)
				if(i->type() == type)
					result << i;
			return result;
		}
		class Level {
			private:
				friend class Indexer;
				int mNum;
				Level(int num) : mNum(num) {}
				QList<const Indexable *> mIndexables;
			public:
				bool operator<(const Level &other) const { return mNum < other.mNum; }
				int num() const { return mNum; }
				//All the indexable things on the level
				const QList<const Indexable *> &indexables() const { return mIndexables; }
				//Filters for special type
				QList<const Indexable *> filter(Indexable::Type type) const { return Indexer::filter(mIndexables, type); }
				QList<const Indexable *> objects() const { return filter(Indexable::Object); }
				QList<const Indexable *> structurals() const { return filter(Indexable::Structural); }
				QList<const Indexable *> vehicles() const { return filter(Indexable::Vehicle); }
		};
	private:
		QList<Indexable *> mIndexables;
		QList<const Level *> mLevels;//List of the levels, sorted by the index
		QHash<int, Level *> mLevelIndex;//Levels as a hash, to support constant access to them
		void insertToLevel(const Indexable *indexable, int level);
		QHash<QString, SemaphoreGroup *> mSemaphoreGroups;
		QTimer *tickTimer;
		QTime lastTime;
		int timeDiff;
		int mSpeed;
		int mTraffic;
	public:
		const QList<const Level *> &levels() const { return mLevels; }//Sorted list of levels
		void insertIndexable(Indexable *indexable);//Place something into it. "eats" the thing - it will be destroyed when the indexer is destroyed
		Indexer &operator<<(Indexable *indexable) {
			insertIndexable(indexable);
			return *this;
		}
		void insertSemaphoreGroup(SemaphoreGroup *group);
		void removeIndexable(Indexable *indexable);
		const QList<Indexable *> &indexables() { return mIndexables; }//All the imdexable objets
		QList<Indexable *> filter(Indexable::Type type) { return filter(mIndexables, type); }//Filtered by type
		QList<Indexable *> vehicles() { return filter(Indexable::Vehicle); }
		QList<Indexable *> invisibles() { return filter(Indexable::Invisible); }
		QList<Point *> points();//Only points
		const QHash<QString, SemaphoreGroup *> &semaphoreGroups() { return mSemaphoreGroups; }
		Indexer();
		~Indexer();
		void setSpeed(int speed);//set emulation speed 0(slow)-2(fast)
		int speed() const { return mSpeed; }
		void setTraffic(int value);//set amount of traffic - 0 = low, 3 = very heavy
		int traffic() const { return mTraffic; }
	signals:
		void tick();//emited ~7-15 times a second (depending on the emulation speed)
		void timeTick(int timeChange);//emited sometimes with difference from the last one
		void preTick(int timeChange);//emited before every timeTick (fer resolving priorities)
		void clearPlan();//Emited when all vehicles are removed, but the plan survives
	private slots:
		void getTimeDelta();
	public slots:
		void start();
		void stop();
		void pause();
};

#endif//LEVELS_H
