/*
	Crossing
	Copyright (C) 2007 Michal Vaner (michal.vaner@kdemail.net)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	he Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "load.h"
#include "crossget.h"
#include "indexer.h"
#include "point.h"
#include "line.h"
#include "globals.h"
#include "semaphore_group.h"
#include "mapobj.h"

#include <QDomDocument>
#include <QFile>
#include <QCoreApplication>
#include <QPixmap>
#include <QDir>
#include <QRegExp>
#include <QHash>

//Bodies for the exceptions

#define EXCP(WHAT) void WHAT::x() {\
}\
\
WHAT::~WHAT() throw(){\
}

EXCP(XMLLoadFormatE);
EXCP(XMLLoadForeignE);
EXCP(XMLLoadFileNotFoundE);
EXCP(XMLLoadDataE);
EXCP(LoadUnknownE);
EXCP(ImageNotFoundE);

inline static QPixmap getImg(const QString &xmlpath, const QString &name) {
	QString final;
	if (QDir(name).isAbsolute())
		final = name;
	else {
		QRegExp filter(QString("^.*%1").arg(QDir::separator()));//Get everything until the last separator
		filter.indexIn(xmlpath);
		final = QDir::cleanPath(filter.cap(0) + name);
	}
	QPixmap pm(final);
	if (pm.isNull())
		throw ImageNotFoundE(final);
	return pm;
}

Indexer *loadXmlFile(const QString &fn, Crossget *crossget) {
	//Some loading
	QFile inf(fn);
	if (!inf.open(QIODevice::ReadOnly | QIODevice::Text))
		throw XMLLoadFileNotFoundE();
	QDomDocument doc;
	if (!doc.setContent(&inf, true))
		throw XMLLoadFormatE();
	//Is it mine?
	if ((doc.documentElement().tagName() != "crossing") || (doc.documentElement().namespaceURI() != NS))
		throw XMLLoadForeignE();
	QDomNodeList backg = doc.elementsByTagNameNS(NS, "background");
	if(!backg.count())
		throw XMLLoadDataE(QCoreApplication::translate("xml_errors", "No background specified"));
	if(backg.count() > 1)
		throw XMLLoadDataE(QCoreApplication::translate("xml_errors", "Too many bacgrounds specified"));
	QPixmap backpix(getImg(fn, backg.item(0).toElement().text()));
	std::auto_ptr<Indexer> indexer(new Indexer);//Make sure auto-deleted if an exception occures
	//The rest of visuals
	QDomNodeList objs = doc.elementsByTagNameNS(NS, "object");
	for(int i = 0; i < objs.count(); ++i) {
		indexer->insertIndexable(new MapObject(getImg(fn, objs.item(i).toElement().text()), objs.item(i).toElement()));
	}
	//Load all the points
	QDomNodeList points = doc.elementsByTagNameNS(NS, "point");
	QHash<QString, Point *> pointNames;
	for(int i = 0; i < points.count(); ++i) {
		Point *p = new Point(points.item(i).toElement(), *indexer);//The indexer will take care of destruction
		if(pointNames.contains(p->id()))
			throw XMLLoadDataE(QCoreApplication::translate("xml_errors", "Point ID conflict"));
		pointNames.insert(p->id(), p);
	}
	QDomNodeList lines = doc.elementsByTagNameNS(NS, "line");
	for(int i = 0; i < lines.count(); ++i) {
		new Line(lines.item(i).toElement(), pointNames, *indexer);//The indexer will take care of destruction
	}
	QDomNodeList sgroups = doc.elementsByTagNameNS(NS, "semaphores");
	for(int i = 0; i < sgroups.count(); ++i)
		new SemaphoreGroup(sgroups.item(i).toElement(), indexer.get());//The indexer will take care of destruction
	foreach(Point *point, indexer->points())
		point->constructStructure();
	//Everything prepared, feed it to the right places..
	crossget->setBackground(backpix);
	crossget->insertIndexer(indexer.get());
	return indexer.release();//OK, do NOT delete now
}
